/* Jungle Monkey
 * Copyright (C) 1999-2001  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef _JMANN_H
#define _JMANN_H

#include <glib.h>
#include <gnet/gnet.h>

#include "util/util.h"
#include "util/elf.h"

typedef struct _JMAnn JMAnn;

struct _JMAnn
{
  gchar*   name;	/* Name	(unscaped)		*/
  gchar*   path;	/* Path to ann (escaped)	*/
  GURL*	   url;		/* URL				*/
  gboolean is_local;	/* Is announcement a local obj  */

  JMAnn*   parent;	/* May have a parent		*/
  GList*   kids;	/* May have children		*/

  Timeval  last_update;	/* Time last update received	*/

  ElfNode* elf;		/* Elf node for reading/writing	*/
};


JMAnn*   jmann_new (const gchar* name);
void	 jmann_delete (JMAnn* ann);

JMAnn*	 jmann_get_child (JMAnn* ann, const gchar* name);
#define  jmann_get_url(A) ((A)->url)

#define  jmann_is_index(A)    elf_get_attribute_bool((A)->elf, "index")
gchar*   jmann_make_path (JMAnn* parent, const gchar* name);

#define  jmann_set_attr(A,N,V)        elf_set_attribute((A)->elf, N, V)
#define  jmann_set_attr_int(A,N,V)    elf_set_attribute_int((A)->elf, N, V)
#define  jmann_set_attr_bool(A,N,V)   elf_set_attribute_bool((A)->elf, N, V)
#define  jmann_set_attr_data(A,N,V)   elf_set_attribute_data((A)->elf, N, V)
void	 jmann_set_attr_url (JMAnn* ann, const gchar* name, const GURL* value);
void	 jmann_set_attr_sha (JMAnn* ann, const gchar* name, const GSHA* value);

#define  jmann_get_attr(A,N)          elf_get_attribute((A)->elf, N)
#define  jmann_get_attr_int(A,N)      elf_get_attribute_int((A)->elf, N)
#define  jmann_get_attr_bool(A,N)     elf_get_attribute_bool((A)->elf, N)
#define  jmann_get_attr_data(A,N,B,L) elf_get_attribute_data((A)->elf, N, B, L)
GURL*	 jmann_get_attr_url (const JMAnn* ann, const gchar* name);
GSHA*	 jmann_get_attr_sha (const JMAnn* ann, const gchar* name);

#endif /* _JMANN_H */
